package com.example.moresecodevisualizer;

import java.io.File;
import java.util.ArrayList;

import android.app.TabActivity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnKeyListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.TabHost;
import android.widget.Toast;
import android.widget.SeekBar.OnSeekBarChangeListener;
/**
 * The main launcher activity for Morse code visualizer application.
 * @author SJSU Students

 *
 */
public class MorseCodeVisualizer extends TabActivity {
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.main);        
        Log.i("MorseCodeVisualizer", "started activity.....");
        TabHost mTabHost = getTabHost();
        mTabHost.addTab(mTabHost.newTabSpec("halfTab").setIndicator("Half Screen").setContent(R.id.halfScreenLayout));
        mTabHost.addTab(mTabHost.newTabSpec("fullTab").setIndicator("Full Screen").setContent(R.id.fullScreenLayout));
        mTabHost.setCurrentTab(0);

        initValues();
        createMorseCodeTable();
        
        OnSeekBarChangeListener sbl = new OnSeekBarChangeListenerImpl();
        sb.setOnSeekBarChangeListener(sbl);
        sbFull.setOnSeekBarChangeListener(sbl);
        
        OnKeyListener okl = new OnKeyListenerImpl();
        et.setOnKeyListener(okl);
        etFull.setOnKeyListener(okl);

        OnClickListener ocl = new OnClickListenerImpl();
        sendButton.setOnClickListener(ocl);
        sendButtonFull.setOnClickListener(ocl);
        msgButton1.setOnClickListener(ocl);
        msgButton2.setOnClickListener(ocl);
        msgButton3.setOnClickListener(ocl);
        
        OnClickListener ocl1 = new OnClickListenerImpl1();
        lf.setOnClickListener(ocl1);
        lh.setOnClickListener(ocl1);
    }
    

    private void initValues()
    {	
        morseCodeStatePref = getSharedPreferences("MorseCodePref", 
        		   Context.MODE_WORLD_WRITEABLE + Context.MODE_WORLD_READABLE);
        
        iv = (ImageView)findViewById(R.id.morseCodeImageView);
        lf = (RelativeLayout)this.findViewById(R.id.fullScreenLayout);
        lh = (LinearLayout)this.findViewById(R.id.halfScreenLayout);
        sb = (SeekBar)this.findViewById(R.id.intensitySlider);
        sbFull = (SeekBar)this.findViewById(R.id.intensitySliderFull);
        sendButton = (Button)this.findViewById(R.id.sendButton);
        sendButtonFull = (Button)this.findViewById(R.id.sendButtonFull);
        msgButton1 = (Button)this.findViewById(R.id.msgButton1);
        msgButton2 = (Button)this.findViewById(R.id.msgButton2);
        msgButton3 = (Button)this.findViewById(R.id.msgButton3);
        et = (EditText) findViewById(R.id.msgTextEditText);
        etFull = (EditText) findViewById(R.id.msgTextEditTextFull);
        msgButton1.setText(morseCodeStatePref.getString("message1", "Message 1"));
        msgButton2.setText(morseCodeStatePref.getString("message2", "Message 2"));
        msgButton3.setText(morseCodeStatePref.getString("message3","Message 3"));        
        
        int progress = morseCodeStatePref.getInt("intensity", 1);		
		intensityBright = WHITE - (ALPHAMUL*progress);		
        sb.setProgress(progress);
        sbFull.setProgress(progress);
        iv.setBackgroundColor(intensityBright);
        lf.setBackgroundColor(intensityBright);
    }
    private void createMorseCodeTable() {        
        File morseTable = new File("/data/data/com.example.moresecodevisualizer/shared_prefs/MORSE_CODE_TABLE.xml");
        Log.i("MorseCodeVisualizer", "Morse Code table exists:" + (new Boolean (morseTable.exists())).toString());
        if (!morseTable.exists())
        {
        	Log.i("MorseCodeVisualizer","Creating table.....");
    		SharedPreferences morsePref = getSharedPreferences("MORSE_CODE_TABLE",		
 				   Context.MODE_WORLD_WRITEABLE + Context.MODE_WORLD_READABLE);
    		Editor morseEditor = morsePref.edit();
    		
    		morseEditor.putString("A", ".-");
    		morseEditor.putString("B", "-..");
    		morseEditor.putString("C", "-.-.");
    		morseEditor.putString("D", "-..");
    		morseEditor.putString("E", ".");
    		morseEditor.putString("F", "..-.");
    		morseEditor.putString("G", "--.");
    		morseEditor.putString("H", "....");
    		morseEditor.putString("I", "..");
    		morseEditor.putString("K", "-.-");
    		morseEditor.putString("L", ".-..");
    		morseEditor.putString("M", "--");
    		morseEditor.putString("N", "-.");
    		morseEditor.putString("O", "---");
    		morseEditor.putString("P", ".--.");
    		morseEditor.putString("Q", "--.-");
    		morseEditor.putString("R", ".-.");
    		morseEditor.putString("S", "...");
    		morseEditor.putString("T", "-");
    		morseEditor.putString("U", "..-");
    		morseEditor.putString("V", "...-");
    		morseEditor.putString("W", ".--");
    		morseEditor.putString("X", "-..-");
    		morseEditor.putString("Y", "-.--");
    		morseEditor.putString("Z", "--..");
    		morseEditor.putString("0", "-----");
    		morseEditor.putString("1", ".----");
    		morseEditor.putString("2", "..---");
    		morseEditor.putString("3", "...--");
    		morseEditor.putString("4", "....-");
    		morseEditor.putString("5", ".....");
    		morseEditor.putString("6", "-....");
    		morseEditor.putString("7", "--...");
    		morseEditor.putString("8", "---..");
    		morseEditor.putString("9", "----.");
    		morseEditor.putString(".", ".-.-.-");
    		morseEditor.putString(",", "--..--");
    		morseEditor.putString("?", "..--..");
    		morseEditor.putString("'", ".----.");
    		morseEditor.putString("!", "-.-.--");
    		morseEditor.putString("/", "-..-.");
    		morseEditor.putString("(", "-.--.");
    		morseEditor.putString(")", "-.--.-");
    		morseEditor.putString("&", ".-...");
    		morseEditor.putString(":", "---...");
    		morseEditor.putString(";", "-.-.-.");
    		morseEditor.putString("=", "-...-");
    		morseEditor.putString("+", ".-.-.");
    		morseEditor.putString("-", "-....-");
    		morseEditor.putString("_", "..--.-");
    		morseEditor.putString("\"", ".-..-.");
    		morseEditor.putString("$", "...-..-");
    		morseEditor.putString("@", ".--.-.");
    		morseEditor.commit();
        }        	
    }
    /**convertToMorseCode() converts text message to Morsecode message
       tokenize the text message delimited by spaces
	   convert each text character to Morse code equivalent (after converting to uppercase)
	   insert intra-text character gap: single space
	   insert inter-text character gap: two spaces
	   re-insert inter-text word gap: three spaces
	   character not found in Morse Code table would be skipped  
	 */
    private StringBuilder convertToMorseCode(String textMessage) {
    	 
    	String[] tokens = textMessage.split(" ");
    	StringBuilder sb = new StringBuilder();
    	SharedPreferences morsePref = getSharedPreferences("MORSE_CODE_TABLE",		
				   Context.MODE_WORLD_WRITEABLE + Context.MODE_WORLD_READABLE);
        for (String s:tokens)
        {
       	
        	for (int i = 0; i < s.length(); i++)
        	{
        		String p = morsePref.getString((s.substring(i,i+1)).toUpperCase(),"Key did not exist");
        		StringBuilder sb1 = new StringBuilder();
        		if ((p.compareTo("Key did not exist"))==0)
        			continue;
        		for (int j = 0; j< p.length()-1; j++)
        		{
        			sb1.append(p.charAt(j));
        			//intra-text character gap: single space
        			sb1.append(" ");
        		}
        		sb1.append((p.charAt(p.length() - 1))+" ");
        		sb.append(sb1);
        		
        		//insert inter-text character gap: second spaces
        		sb.append(" ");
        	}
        	//insert inter-text word gap: third space
        	sb.append(" ");
        }
        //delete extra 3 spaces at the end of message
        sb.delete(sb.length()-3, sb.length());
        return sb;
    }
    /**
      convertToFlashes() scans the Morse code message and 
      creates an array of flash durations and intensity for each duration
     */
    private void convertToFlashes(StringBuilder morseCodeMessage) {
    	
    	ArrayList<Long> temp = new ArrayList<Long>();
    	flashDuration = new ArrayList<Long>();
        intensity = new ArrayList<Integer>();
        long ms1 = 500;
        long ms2 = 1500;
        long ms3 = 3500;
        long sum;
        
        Log.i("MorseCodeVisualizer","Converting to Flashes now..");
   		for (int i=0; i < morseCodeMessage.length();)
   		{
   			//Log.i("MorseCodeVisualizer","char " + i + ":" + morseCodeMessage.charAt(i));
   			switch (morseCodeMessage.charAt(i))
   			{
   				case '.':
   					intensity.add(intensityBright);
   					temp.add(ms1);
   					i++;
   					break;
   				case '-':
   					temp.add(ms2);
   					intensity.add(intensityBright);
   					i++;
   					break;
   				case ' ':
   					//iv.setBackgroundColor(intensityDark);
   					if(morseCodeMessage.charAt(i+1)==' ')
   					{
   						if(morseCodeMessage.charAt(i+2)==' ')
   						{
   							//inter-text word gap
   							temp.add(ms3);
   	   						intensity.add(intensityDark);
   	   						i+=3;
   						}
   						else
   						{   							
   							//inter-text character gap
   							temp.add(ms2);
   							intensity.add(intensityDark);
   							i+=2;
   						}
   					}
   					else
   					{   						
   						//intra-text character gap
   						temp.add(ms1);
   						intensity.add(intensityDark);   					
   						i++;
   					}
   					break;
   				default:
   					i++;
   					break;
   			}
   		}
   		
   		/*
   		for (int k = 0; k<temp.size();k++)
   			Log.i("MorseCodeVisualizer","Flash Duration "+k + ":"+(new Long(temp.get(k))).toString());	
   		*/
   		flashDuration.add(0, new Long(0));
   		flashDuration.add(1, temp.get(0));
   		sum = temp.get(0) + temp.get(1);
   		for (int i=2;i<temp.size();i++)
   		{
   			flashDuration.add(sum);
   			sum = sum + temp.get(i);	
   		}
   		lastFlash = temp.get((temp.size())-1);

    }
    private void sendFlashes() {    	
    	
    	Log.i("MorseCodeVisualizer","sending flashes to the handler....");
    	
    	long upm = SystemClock.uptimeMillis();
   		for (int j=0;j<flashDuration.size();j++)
   			//the first flash will START after 1 second
   			flashDuration.set(j, (flashDuration.get(j)+upm+1000));
   		
   		for (int i = 0; i < flashDuration.size(); i++)
        	handler.postAtTime(new FlashTask((int)(intensity.get(i))), (Long)(flashDuration.get(i)));
    }
    /**
     * 
     * storeString() stores the most recently sent message to database,
     *  deletes the least recently used one 
     */
    private void storeMessage(String storeString) {
    	Editor morsecodeStateEditor = morseCodeStatePref.edit();
    	Boolean done = false;
    	Log.i("MorseCodeVisualizer","Storing the message....");
    	for(int j=1; j<=3; j++)
        {
        	String s = morseCodeStatePref.getString("message"+j,"String Not Found");
           	if (s.compareTo("String Not Found")==0)
           	{
           		morsecodeStateEditor.putString("message"+j, storeString);
           		morsecodeStateEditor.commit();
           		done = true;
           		break;
           	}
        }
        if (!done)
        {
        	morsecodeStateEditor.remove("message1");
           	morsecodeStateEditor.putString("message1",morseCodeStatePref.getString("message2", "NO"));
            morsecodeStateEditor.putString("message2",morseCodeStatePref.getString("message3", "NO"));
            morsecodeStateEditor.putString("message3", storeString);
           	morsecodeStateEditor.commit();
        }                   
        msgButton1.setText(morseCodeStatePref.getString("message1", "Message 1"));
        msgButton2.setText(morseCodeStatePref.getString("message2", "Message 2"));
        msgButton3.setText(morseCodeStatePref.getString("message3", "Message 3"));
    }
    
    private final int WHITE=0XFFFFFFFF, ALPHAMUL=0X33000000;    
    private int intensityBright = 0xFFFFFFFF;
    private int intensityDark = 0x33FFFFFF;
    private ImageView iv;
    private LinearLayout lh;
    private RelativeLayout lf;
    private SeekBar sb, sbFull;
    private Button sendButton, sendButtonFull, msgButton1, msgButton2, msgButton3;
    private EditText et, etFull;
    private SharedPreferences morseCodeStatePref;
    private Handler handler = new Handler();
    private ArrayList<Long> flashDuration;
    private ArrayList<Integer> intensity;
    private long lastFlash;
    private class OnSeekBarChangeListenerImpl implements OnSeekBarChangeListener {
		public void onProgressChanged(SeekBar seekBar, int progress,
				boolean fromUser) {
			
			// vary alpha component of intensityBright			
			intensityBright = WHITE - (ALPHAMUL*progress);
			sb.setProgress(progress);
			sbFull.setProgress(progress);
			iv.setBackgroundColor(intensityBright);
			lf.setBackgroundColor(intensityBright);
			
			// store state
			Editor statePrefEditor = morseCodeStatePref.edit();    		
			statePrefEditor.putInt("intensity", progress);
    		statePrefEditor.commit();
    	}
		
    	public void onStartTrackingTouch(SeekBar seekBar) {	
    		//we're not doing anything here
		}

		public void onStopTrackingTouch(SeekBar seekBar) {
    		//we're not doing anything here
		}
    }    
    private class OnClickListenerImpl implements OnClickListener {
    	public void onClick(View v) {

    		
    			
    		
    		String textMessage;
    		if (v.getId() == R.id.sendButton)
    		{
    			//half screen send button
    			textMessage = (et.getText().toString());
    		}
    		else
    		{
    			// full screen send or messages button
    			if (v.getId() == R.id.sendButtonFull)
    				textMessage = (etFull.getText().toString());
    			else
    			{
    				String key = "";
    				morseCodeStatePref = getSharedPreferences("MorseCodePref", 
    		         		   Context.MODE_WORLD_WRITEABLE + Context.MODE_WORLD_READABLE);
    				switch(v.getId())
        			{    				
    				case R.id.msgButton1:
    					key = "message1";
    					break;
    				case R.id.msgButton2:
    					key = "message2";
    					break;
    				case R.id.msgButton3:
    					key = "message3";
    					break;
        			}
    				textMessage = morseCodeStatePref.getString(key, "Message 1");
    			}
    		}
    		
    		if (textMessage.compareTo("")==0)
    		{
    			Toast.makeText(MorseCodeVisualizer.this, "Please input message!", Toast.LENGTH_SHORT).show();
    		}
    		else
    		{
    		//conversion
    		Log.i("MorseCodeVisualizer","converting text to Morse: " + textMessage);
       		StringBuilder morseCodeMessage = convertToMorseCode(textMessage);
       		Log.i("MorseCodeVisualizer","converted text to Morse: " + morseCodeMessage.toString());
       		convertToFlashes(morseCodeMessage);
       		Log.i("MorseCodeVisualizer","converted Morse to flashes, Number of flashes: "+ (new Integer(flashDuration.size())).toString());
       		
       		//send message flashes
       		if (v.getId() == R.id.sendButton)
       		{
       			sendFlashes();
       		}
       		else
       		{
       			Intent in = new Intent(MorseCodeVisualizer.this, FullView.class);
            	Bundle bundle = new Bundle();
            	
            	long[] l = new long[flashDuration.size()];
            	for (int i=0;i<flashDuration.size();i++)
            	{
            		l[i]=flashDuration.get(i);
            	}
            	
            	bundle.putIntegerArrayList("intensityArray", intensity);
            	bundle.putLongArray("flashDurationArray", l);
            	bundle.putLong("lastFlashDuration", lastFlash);
            	  
            	in.putExtras(bundle);
          	  	MorseCodeVisualizer.this.startActivity(in);
       		}
       		       		
       		//store the message
       		storeMessage(textMessage);
    		}
    	}
    }
    private class OnClickListenerImpl1 implements OnClickListener {
    	public void onClick(View v) {
    		InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(lh.getWindowToken(), 0);
            imm.hideSoftInputFromWindow(lf.getWindowToken(), 0);
            if(v.getId()==R.id.halfScreenLayout)
          	  etFull.setText((et.getText()).toString());
            else
          	  et.setText((etFull.getText()).toString());
    	}
    }
    private class OnKeyListenerImpl implements OnKeyListener {
    	public boolean onKey(View v, int keyCode, KeyEvent event) {
    	    if (keyCode == KeyEvent.KEYCODE_ENTER) {
              InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
              imm.hideSoftInputFromWindow(et.getWindowToken(), 0);
              //Toast.makeText(MorseCodeVisualizer.this, et.getText(), Toast.LENGTH_SHORT).show();
              if(v.getId()==R.id.msgTextEditText)
            	  etFull.setText((et.getText()).toString());
              else
            	  et.setText((etFull.getText()).toString());
				  return true;
            }
            return false;
    	}
    }
	private class FlashTask implements Runnable {

		public FlashTask(int c){
			this.intensity = c;
		}
		public void run(){
			iv.setBackgroundColor(intensity);
			Log.i("MorseCodeVisualizer","Flashtask flashing!!!");
		}
		private int intensity;
	}
}